#ifndef DENSEMATRIX_H
#define DENSEMATRIX_H

#include <cassert>

template<class T>
struct DenseMatrix
{
   unsigned int n;    // dimension (n*n matrix)
   T *data;           // entries stored column by column (FORTRAN layout)
   int *ipiv;         // pivot permutation for use by LAPACK

   DenseMatrix(void)
      : n(0), data(0), ipiv(0)
   {}

   explicit DenseMatrix(unsigned int n_)
      : n(n_), data(new T[n_*n_]), ipiv(new int[n_])
   {
      for(unsigned int i=0; i<n*n; ++i)
         data[i]=0;
   }

   ~DenseMatrix(void)
   {
      delete[] data; data=0;
      delete[] ipiv; ipiv=0;
   }

   T &operator()(unsigned int i, unsigned int j)
   {
      assert(0<=i && i<n && 0<=j && j<n);
      return data[i+n*j];
   }

   const T &operator()(unsigned int i, unsigned int j) const
   {
      assert(0<=i && i<n && 0<=j && j<n);
      return data[i+n*j];
   }
};

typedef DenseMatrix<float> DenseMatrixf;
typedef DenseMatrix<double> DenseMatrixd;

// these solve A*x=b, storing x in b and destroying A in the process.
void destructive_solve(DenseMatrixf &A, float *b);
void destructive_solve(DenseMatrixd &A, double *b);

#endif
