#include <cstdio>
#include <cstring>
#include "springs.h"
#include "densematrix.h"

using namespace std;

void Springs::
read_config(const char *filename)
{
   FILE *fp=fopen(filename, "r");

   fscanf(fp, "%f", &final_time);
   fscanf(fp, "%f", &frame_rate);
   fscanf(fp, "%f", &stiffness);
   fscanf(fp, "%f", &damping);
   fscanf(fp, "%f", &rest_length);
   fscanf(fp, "%f", &mass);
   fscanf(fp, "%d", &num_springs);

   x0=new Vec3f[num_springs+1];
   v0=new Vec3f[num_springs+1];

   for(int i=0; i<=num_springs; ++i){
      fscanf(fp, "%f %f %f", &x0[i][0], &x0[i][1], &x0[i][2]);
      fscanf(fp, "%f %f %f", &v0[i][0], &v0[i][1], &v0[i][2]);
   }

   fclose(fp);
}

static void write_frame(FILE *fp, int n, Vec3f *x)
{
   for(int i=0; i<n; ++i)
      fprintf(fp, "%g %g %g\n", x[i][0], x[i][1], x[i][2]);
   fprintf(fp, "\n");
}

void Springs::
run(TimeIntegration method, const char *output_filename)
{
   switch(method){
      case FORWARD_EULER: init_forward_euler(); break;
      case SYMPLECTIC_EULER: init_symplectic_euler(); break;
      case BACKWARD_EULER: init_backward_euler(); break;
      case IMPLICIT_COMPROMISE: init_implicit_compromise(); break;
   }

   FILE *fp=fopen(output_filename, "w");
   fprintf(fp, "%d\n", num_springs+1); // write out the number of particles

   float time=0;
   write_frame(fp, num_springs+1, x);

   float dt=1/frame_rate;

   if(method==SYMPLECTIC_EULER)
      half_step_symplectic_euler(dt);

   while(time<final_time){
      switch(method){
         case FORWARD_EULER: step_forward_euler(dt); break;
         case SYMPLECTIC_EULER: step_symplectic_euler(dt); break;
         case BACKWARD_EULER: step_backward_euler(dt); break;
         case IMPLICIT_COMPROMISE: step_implicit_compromise(dt); break;
      }
      time+=1/frame_rate;
      write_frame(fp, num_springs+1, x);
   }

   fclose(fp);
}

//===================================== Force Computations =====================
// Note: you may want to add functions that are common to the implicit methods

void Springs::
compute_spring_forces(const Vec3f *x, const Vec3f *v, Vec3f *f)
{
   memset(f, 0, (num_springs+1)*sizeof(Vec3f));  // zero out forces to start
   //@@@ add code here to compute the forces on each particle (storing them in f)
   //@@@ given the positions and velocities passed in as arguments
}

//===================================== Forward Euler =====================
// Note: I've already implemeted this for you

void Springs::
init_forward_euler()
{
   // allocate space for x, v, and f (nothing else needed)
   x=new Vec3f[num_springs+1];
   v=new Vec3f[num_springs+1];
   f=new Vec3f[num_springs+1];
   // copy in initial conditions
   memcpy(x, x0, (num_springs+1)*sizeof(Vec3f));
   memcpy(v, v0, (num_springs+1)*sizeof(Vec3f));
}

void Springs::
step_forward_euler(float dt)
{
   compute_spring_forces(x, v, f);
   for(int i=0; i<=num_springs; ++i){
      x[i]+=dt*v[i];
      v[i]+=(dt/mass)*f[i];
   }
}

//===================================== Symplectic Euler =====================

void Springs::
init_symplectic_euler()
{
   //@@@ add code to initialize anything you need for staggered symplectic euler
}

void Springs::
half_step_symplectic_euler(float dt)
{
   //@@@ add code to advance just the velocity by half a step to get the staggering working
}

void Springs::
step_symplectic_euler(float dt)
{
   //@@@ add code to advance by dt with staggered symplectic Euler
}

//===================================== Backward Euler =====================

void Springs::
init_backward_euler()
{
   //@@@ add code to initialize anything you need for backward Euler
}

void Springs::
step_backward_euler(float dt)
{
   //@@@ add code to advance by dt with backward Euler
}

//===================================== Implicit Compromise =====================

void Springs::
init_implicit_compromise()
{
   //@@@ add code to initialize anything you need for implicit compromise
}

void Springs::
step_implicit_compromise(float dt)
{
   //@@@ add code to advance by dt with implicit compromise
}

