#ifndef SPRINGS_H
#define SPRINGS_H

#include "vec3.h"

enum TimeIntegration {FORWARD_EULER, SYMPLECTIC_EULER, BACKWARD_EULER, IMPLICIT_COMPROMISE};

struct Springs{
   // configuration
   float final_time;
   float frame_rate;
   float stiffness;
   float damping;
   float rest_length;
   float mass;
   int num_springs;
   Vec3f *x0, *v0;

   Springs()
      : final_time(0), frame_rate(24), stiffness(1), damping(1),
        rest_length(1), mass(1), num_springs(0), x0(0), v0(0),
	x(0), v(0), f(0)
   {}

   ~Springs()
   {
      delete[] x0; x0=0;
      delete[] v0; v0=0;
      delete[] x; x=0;
      delete[] v; v=0;
      delete[] f; f=0;
   }

   void read_config(const char *filename);
   void run(TimeIntegration method, const char *output_filename);

   private:

   Vec3f *x, *v; // the current positions and velocities (what gets written out to a file)
   Vec3f *f;     // something to compute forces in
   //@@@ you may want to add additional arrays that are used by the time integrators
   //@@@ but be sure to initialize them to empty in the Springs() constructor and delete[] them in
   //@@@ in the ~Springs() destructor.

   void compute_spring_forces(const Vec3f *x, const Vec3f *v, Vec3f *f);
   void init_forward_euler();
   void step_forward_euler(float dt);
   void init_symplectic_euler();
   void half_step_symplectic_euler(float dt);
   void step_symplectic_euler(float dt);
   void init_backward_euler();
   void step_backward_euler(float dt);
   void init_implicit_compromise();
   void step_implicit_compromise(float dt);
};

#endif

