#include <cstdio>
#include <iostream>
#include <fstream>
#include "gluvi.h"
#include "vec3.h"

using namespace std;

char frame_number[100]="frame 0";
unsigned int frame=0;
unsigned int n;
vector<vector<Vec3f> > x;

void read(const char *filename)
{
   ifstream input(filename);
   input>>n;
   for(unsigned int f=0; ; ++f){
      Vec3f x0;
      input>>x0;
      if(!input)
	 break;
      x.resize(f+1);
      x[f].resize(n);
      x[f][0]=x0;
      for(unsigned int i=1; i<n; ++i)
	 input>>x[f][i];
   }
   if(x.size()==0){
      exit(1);
   }
}

void set_view(Gluvi::Target3D &cam)
{
   cam.dist=3;
}

void display(void)
{
   unsigned int i;

   glDisable(GL_LIGHTING);
   glColor3f(1, 1, 1);
   glBegin(GL_LINE_STRIP);
   for(i=0; i<n; ++i)
      glVertex3fv(x[frame][i].v);
   glEnd();

   glEnable(GL_LIGHTING);
   GLfloat ambientlight[4]={.3, .3, .3, 1};
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientlight);
   GLfloat lightcolor[4]={1, 1, 1, 1};
   glLightfv(GL_LIGHT0, GL_DIFFUSE, lightcolor);
   glLightfv(GL_LIGHT0, GL_SPECULAR, lightcolor);
   GLfloat lightpos[4]={100, 100, 100, 1};
   glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
   glEnable(GL_LIGHT0);

   GLfloat ambient[4]={0, 0, .5, 1}, diffuse[4]={0, 1, 0, 1}, specular[4]={1, 0, 0, 1};
   glMaterialfv(GL_FRONT, GL_AMBIENT, ambient);
   glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
   glMaterialf(GL_FRONT, GL_SHININESS, 32);

   glMatrixMode(GL_MODELVIEW);
   for(i=0; i<n; ++i){
      glPushMatrix();
      glTranslatef(x[frame][i][0], x[frame][i][1], x[frame][i][2]);
      glutSolidSphere(0.1, 8, 8);
      glPopMatrix();
   }
}

struct CameraButton : public Gluvi::Button {
   CameraButton(const char *t) : Gluvi::Button(t) {}
   void action()
   {
      ofstream camera_output("springs_camera.rib");
      Gluvi::camera->export_rib(camera_output);
   }
};

struct ScreenShotButton : public Gluvi::Button{
   const char *filename_format;
   ScreenShotButton(const char *label, const char *filename_format_) : Gluvi::Button(label), filename_format(filename_format_) {}
   void action()
   { Gluvi::ppm_screenshot(filename_format, frame); }
};

void special_key_handler(int key, int mousex, int mousey)
{
   switch(key){
      case GLUT_KEY_LEFT:
         if(frame>0){
            --frame;
            sprintf(frame_number, "frame %d", frame);
            glutPostRedisplay();
         }
         break;
      case GLUT_KEY_RIGHT:
         if(frame+1<x.size()){
            ++frame;
            sprintf(frame_number, "frame %d", frame);
            glutPostRedisplay();
         }
         break;
      default:
         ;
   }
}

int main(int argc, char **argv)
{
   Gluvi::init("opengl_springs", &argc, argv);

   if(argc!=2){
      cerr<<"Expecting one argument: animation filename"<<endl;
      return 1;
   }

   read(argv[1]);

   glutSpecialFunc(special_key_handler);

   Gluvi::Target3D cam;
   set_view(cam);
   Gluvi::camera=&cam;

   Gluvi::userDisplayFunc=display;

   Gluvi::StaticText frametext(frame_number);
   Gluvi::root.list.push_back(&frametext);

   Gluvi::StaticText keytext("use left/right arrows for frame control");
   Gluvi::root.list.push_back(&keytext);

   CameraButton cambutton("Export camera to RIB");
   Gluvi::root.list.push_back(&cambutton);

   char ppmfileformat[strlen(argv[1])+5];
   sprintf(ppmfileformat, "%s.ppm", argv[1]);
   ScreenShotButton screenshot("Screenshot", ppmfileformat);
   Gluvi::root.list.push_back(&screenshot);

   Gluvi::run();
   return 0;
}

