#include <cstdio>
#include <iostream>
#include <fstream>
#include <vector>
#include "vec3.h"

using namespace std;

using namespace std;

char frame_number[100]="frame 0";
unsigned int frame=0;
unsigned int n;
vector<vector<Vec3f> > x;

bool read(const char *filename, vector<vector<Vec3f> > &x)
{
   ifstream input(filename);
   input>>n;
   for(unsigned int f=0; ; ++f){
      Vec3f x0;
      input>>x0;
      if(!input)
	 break;
      x.resize(f+1);
      x[f].resize(n);
      x[f][0]=x0;
      for(unsigned int i=1; i<n; ++i)
	 input>>x[f][i];
   }
   if(x.size()==0)
      return false;
   else
      return true;
}

void read_camera(const char *camfile, char *&cam)
{
   ifstream in(camfile);
   vector<char> camvec;
   for(;;){
      char c=in.get();
      if(in.good()) camvec.push_back(c);
      else break;
   }
   cam=new char[camvec.size()+1];
   unsigned int i;
   for(i=0; i<camvec.size(); ++i)
      cam[i]=camvec[i];
   cam[i]=0;
}

void output_frame(const char *animation_name, unsigned int frame, const char *camera, vector<vector<Vec3f> > &x)
{
   // figure out the RIB file name, and open it
   char *filename=new char[strlen(animation_name)+20];
   sprintf(filename, "%s%04d.rib", animation_name, frame);
   ofstream out(filename);

   // first line: what image file this RIB file should produce
   sprintf(filename, "%s%04d.tiff", animation_name, frame);
   out<<"Display \""<<filename<<"\" \"file\" \"rgb\""<<endl;
   delete[] filename;

   // next line: image format (width and height in pixels, pixel aspect ratio)
   out<<"Format 640 480 1"<<endl;

   // sampling rate - 2x2 isn't too bad
   out<<"PixelSamples 2 2"<<endl;

   // correct for a monitor with gamma 2.2 (we won't be compositing these images!)
   out<<"Exposure 1.0 2.2"<<endl;

   // then write out the camera specification
   out<<camera<<endl;

   // start the scene
   out<<"WorldBegin"<<endl;

   // example of setting up some lights
   // You may want to change this
   out<<"LightSource \"ambientlight\" 1 \"intensity\" .1 \"lightcolor\" [1 1 1]"<<endl;
   out<<"LightSource \"distantlight\" 2 \"intensity\" 0.8 \"from\" [-5 30 20] \"to\" [0 0 0]"<<endl;
   out<<"LightSource \"distantlight\" 2 \"intensity\" 0.3 \"from\" [-5 -10 20] \"to\" [0 0 0]"<<endl;

   // example of setting up a material for springs
   // You may want to change this. (And you're not limited to just one material of course; you may
   // change materials for different parts of the geometry)
   out<<"Color [0.8 0.8 1]"<<endl;
   out<<"Surface \"plastic\" \"Ka\" 0.1 \"specularcolor\" [1 1 1]"<<endl;

   // output the geometry
   // @@@ Here you need to add code to output RenderMan RIB commands that will draw some representation
   // @@@ of the springs.
   // @@@ For example, you could draw spheres for the particles, then a string of smaller spheres equally
   // @@@ spaced along each spring, or a cylinder for the springs, or anything else reasonable.
   // @@@ Note that the particles in this frame have positions x[frame][0], ... x[frame][n-1], and
   // @@@ the springs connect particle 0 to 1, 1 to 2, ..., n-2 to n-1.

   // finish the scene
   out<<"WorldEnd"<<endl;
}

int main(int argc, char **argv)
{
   if(argc!=3){
      cerr<<"Expecting two arguments: <animation_filename> <camera_filename>"<<endl;
      return 1;
   }

   vector<vector<Vec3f> > x;
   cout<<"reading animation file..."<<endl;
   if(!read(argv[1], x)){
      cerr<<"couldn't read animation file "<<argv[1]<<endl;
      return 1;
   }

   cout<<"reading camera..."<<endl;
   char *camera=0;
   read_camera(argv[2], camera);

   for(unsigned int frame=0; frame<x.size(); ++frame){
      cout<<"writing RIB file for frame "<<frame<<"..."<<endl;
      output_frame(argv[1], frame, camera, x);
   }

   return 0;
}

